@@ -11,17 +11,40 @@ class Placeholder(models.Model):
     slot = models.CharField(_("slot"), max_length=50, db_index=True, editable=False)
     default_width = models.PositiveSmallIntegerField(_("width"), null=True, editable=False)
 
-    def __unicode__(self):
-        return self.slot
-
     class Meta:
         app_label = 'cms'
 
-    def has_change_permission(self, request):
-        opts = self._meta
+    def __unicode__(self):
+        return self.slot
+        
+    def _get_permission(self, request, key):
+        """
+        Generic method to check the permissions for a request for a given key,
+        the key can be: 'add', 'change' or 'delete'.
+        """
         if request.user.is_superuser:
             return True
-        return request.user.has_perm(opts.app_label + '.' + opts.get_change_permission())
+        found = False
+        # check all attached models for change permissions
+        for model in self._get_attached_models():
+            opts = model._meta
+            perm_accessor = getattr(opts, 'get_%s_permission' % key)
+            perm_code = '%s.%s' % (opts.app_label, perm_accessor())
+            # if they don't have the permission for this attached model, bail out
+            if not request.user.has_perm(perm_code):
+                return False
+            else:
+                found = True
+        return found
+
+    def has_change_permission(self, request):
+        return self._get_permission(request, 'change')
+
+    def has_add_permission(self, request):
+        return self._get_permission(request, 'add')
+
+    def has_delete_permission(self, request):
+        return self._get_permission(request, 'delete')
 
     def render(self, context, width):
         from cms.plugin_rendering import render_placeholder
@@ -37,6 +60,18 @@ class Placeholder(models.Model):
             return reduce(operator.add, media_classes)
         return Media()
     
+    def _get_attached_fields(self):
+        """
+        Returns an ITERATOR of all non-cmsplugin reverse foreign key related fields.
+        """
+        from cms.models import CMSPlugin
+        for rel in self._meta.get_all_related_objects():
+            if isinstance(rel.model, CMSPlugin):
+                continue
+            field = getattr(self, rel.get_accessor_name())
+            if field.count():
+                yield rel.field
+    
     def _get_attached_field(self):
         from cms.models import CMSPlugin
         if not hasattr(self, '_attached_field_cache'):
@@ -60,6 +95,13 @@ class Placeholder(models.Model):
         if field:
             return field.model
         return None
+    
+    def _get_attached_models(self):
+        """
+        Returns a list of models of attached to this placeholder.
+        """
+        return [field.model for field in self._get_attached_fields()]
+        
 
     def get_plugins_list(self):
         return list(self.get_plugins())
