@@ -8,20 +8,23 @@ from cms.plugins.file.models import File
 from cms.plugins.googlemap.models import GoogleMap
 from cms.plugins.inherit.models import InheritPagePlaceholder
 from cms.plugins.text.models import Text
-from cms.plugins.text.utils import plugin_tags_to_id_list, \
-    plugin_tags_to_admin_html
-from cms.test.testcases import CMSTestCase, URL_CMS_PAGE, URL_CMS_PAGE_ADD, \
-    URL_CMS_PLUGIN_ADD, URL_CMS_PLUGIN_EDIT, URL_CMS_PAGE_CHANGE, \
-    URL_CMS_PLUGIN_REMOVE
+from cms.plugins.text.utils import (plugin_tags_to_id_list, 
+    plugin_tags_to_admin_html)
+from cms.test.testcases import (CMSTestCase, URL_CMS_PAGE, URL_CMS_PAGE_ADD, 
+    URL_CMS_PLUGIN_ADD, URL_CMS_PLUGIN_EDIT, URL_CMS_PAGE_CHANGE, 
+    URL_CMS_PLUGIN_REMOVE)
+from cms.test.util.context_managers import SettingsOverride
 from django.conf import settings
 from django.contrib.auth.models import User
 from django.core.files.uploadedfile import SimpleUploadedFile
+from django.core.urlresolvers import reverse
 from django.forms.widgets import Media
 from django.template import RequestContext
 from testapp.pluginapp.models import Article, Section
 from testapp.pluginapp.plugins.manytomany_rel.models import ArticlePluginModel
 import os
 
+
     
 
 
@@ -83,8 +86,10 @@ class PluginsTestBaseCase(CMSTestCase):
         request = super(PluginsTestBaseCase, self).get_request(*args, **kwargs)
         request.placeholder_media = Media()
         return request
-        
+
+
 class PluginsTestCase(PluginsTestBaseCase):
+        
 
     def test_01_add_edit_plugin(self):
         """
@@ -287,12 +292,8 @@ class PluginsTestCase(PluginsTestBaseCase):
             'parent': int(response.content)
         }
         response = self.client.post(URL_CMS_PLUGIN_ADD, plugin_data)
-        
-        plugin_data = {
-            'plugin_id': int(response.content)
-        }
-        response = response.client.post(URL_CMS_PLUGIN_REMOVE, plugin_data)
-        self.assertEquals(response.status_code, 200)
+        # no longer allowed for security reasons
+        self.assertEqual(response.status_code, 404)
         
     def test_07_register_plugin_twice_should_raise(self):
         number_of_plugins_before = len(plugin_pool.get_all_plugins())
